// CEOI 2013 - Task: Splot - Checker 
//
// Checker to be used by evaluator.
//
// Usage: splot_checker input_file official_output_file contestants_output_file
//

#include <algorithm>
#include <cassert>
#include <cctype>
#include <cstdlib>
#include <cstring>
#include <fstream>
#include <iostream>
#include <queue>
#include <string>
#include <utility>
#include <vector>
#include <string>
#include <stack>

using namespace std;

#define MAXN 1000005

// point distribution
const double correct_cnt = 0.8;

// official input
string input;

// official output
int cnt_official;
string sol_official;

// contestants data
int cnt_contestant;
string sol_contestant;

int next_node;
stack< int > stks;
stack< int > stkt;
vector< int > G[MAXN];
bool bio[MAXN];
char state[MAXN];

bool endP(string s) { return s == "|o#" || s == "|x#"; }
bool endS(string s) { return s == "#"; }

void add_edge(int a, int b) {
  //cerr << "edge " << a << " " << b << endl;
  G[a].push_back(b);
  G[b].push_back(a);
}

void build_graph() {
  for (int i = 0; i < (int)sol_contestant.size(); ) {
    //cerr << "ide " << sol_contestant.substr(i) << endl;
    if (endP(sol_contestant.substr(i, 3))) {
      //cerr << "endP" << endl;
      int s2 = stks.top(); stks.pop();
      int s1 = stks.top(); stks.pop();
      int t2 = stkt.top(); stkt.pop();
      int t1 = stkt.top(); stkt.pop();

      add_edge(stks.top(), s1);
      add_edge(stks.top(), s2);
      add_edge(t1, next_node);
      add_edge(t2, next_node);

      stkt.push(next_node);
      state[next_node] = sol_contestant[i + 1];
      next_node += 1;

      i += 3;
    }
    else if (endS(sol_contestant.substr(i, 1))) {
      //cerr << "endS" << endl;
      int s2 = stks.top(); stks.pop();
      int s1 = stks.top(); stks.pop();
      int t2 = stkt.top(); stkt.pop();
      int t1 = stkt.top(); stkt.pop();

      add_edge(t1, s2);

      stks.push(s1);
      stkt.push(t2);

      i += 1;
    }
    else if (sol_contestant[i] == 'P') {
      //cerr << "startP" << endl;
      stks.push(next_node);
      state[next_node] = sol_contestant[i + 1];
      next_node += 1;
      i += 2;
    }
    else if (sol_contestant[i] == 'S') {
      //cerr << "startS" << endl;
      i += 1;
    }
    else if (sol_contestant[i] == 'o' || sol_contestant[i] == 'x') {
      //cerr << "ox" << endl;
      stks.push(next_node);
      stkt.push(next_node);
      state[next_node] = sol_contestant[i];
      next_node += 1;
      i += 1;
    }
    else if (sol_contestant[i] == '|') {
      //cerr << "bar" << endl;
      i += 1;
    }
    else {
      //cerr << "bad bad bad" << endl;
      assert(false);
    }
  }
}

int dfs(int n) {
  bio[n] = 1;
  if (state[n] == 'x') return 1;
  
  int ret = 0;

  for (int i = 0; i < (int)G[n].size(); ++i)
    if (!bio[G[n][i]])
      ret += dfs(G[n][i]);

  return ret;
}

int main(int argc, char *argv[])
{
  assert(argc == 4);

  ifstream finput(argv[1]);
  ifstream fofficial(argv[2]);
  ifstream foutput(argv[3]);

  assert(!finput.fail() && !foutput.fail() && !fofficial.fail());

  // Read official input
  if (!(finput >> input)) {
    cout << 0 << endl << "Error reading the input file: cannot read the first and only line." << endl;
    exit(0);
  }  

  // Read official output
  if (!(fofficial >> cnt_official)) {
    cout << 0 << endl << "Error reading the official output file: cannot read cnt_official." << endl;
    exit(0);
  }
  if (!(fofficial >> sol_official)) {
    cout << 0 << endl << "Error reading the official output file: cannot read sol_official." << endl;
    exit(0);
  }

  // Read contestant's output

  if (!(foutput >> cnt_contestant)) {
    cout << 0 << endl << "Error reading the output file: cannot read cnt_contestant." << endl;
    exit(0);
  }
  if (!(foutput >> sol_contestant)) {
    sol_contestant = "";
  }
  string crap;
  if (foutput >> crap) {
    cout << 0 << endl << "Error reading the output file: too much data at the end of file." << endl;
    exit(0);
  }

  // Check the solution.

  if (cnt_contestant != cnt_official) {
    cout << 0 << endl << "Incorrect output: wrong number of cars. " << 
      "Contestants says " << cnt_contestant << ", should be " << cnt_official << "." << endl;
    exit(0);
  }
  if (sol_contestant.size() == 0) {
    cout << correct_cnt << endl << "Correct number of cars, but no configuration in the output." << endl;
    exit(0);
  }
  if (sol_contestant.size() != sol_official.size()) {
    cout << correct_cnt << endl << "Correct number of cars, but the solution string is wrong size." << endl;
    exit(0);
  }

  int cnt_x = 0;

  for (int i = 0; i < (int)sol_contestant.size(); ++i) {
    if (sol_contestant[i] == 'x')
      ++cnt_x;
    if (sol_contestant[i] != input[i]) {
      if (input[i] == 'o' && sol_contestant[i] == 'x') { // only good condition	
	continue;
      }
      cout << correct_cnt << endl << "Correct final count, but the solution string is wrong! " << 
	"Invalid character at index " << i << "." << endl;
      exit(0);
    }
  }

  if (cnt_x != cnt_contestant) {
    cout << correct_cnt << endl << "Correct final count, but the output configuration is wrong (incorrect number of cars)." << endl;
    exit(0);
  }

  build_graph();

  memset(bio, 0, sizeof bio);
  int cnt_dfs = dfs(stks.top());

  if (cnt_dfs != cnt_contestant) {
    cout << correct_cnt << endl << "Correct final count, but the output configuration is wrong (some cars are blocked)." << endl;
    exit(0);
  }

  cout << 1 << endl << "Correct!" << endl;

  return 0;
}
