// CEOI 2013 - Task: Treasure - Test server
//
// Test server to be used by the competitor during the contest.
//
// Usage: treasure_test ./my_solution input_file
//
// Detailed execution log written to treasure.log.
// Important messages written to the standard output.

#include <algorithm>
#include <cassert>
#include <cctype>
#include <cstdlib>
#include <fstream>
#include <iostream>
#include <queue>
#include <string>
#include <utility>
#include <unistd.h>

using namespace std;

#define f1 N*N*N*N
#define f4 (double)3/4*N*N*N*N
#define f8 (double)7/16*N*N*N*N+2*N*N*N
#define f10 (double)7/16*N*N*N*N+N*N
float scores[] = {0.1, 0.4, 0.8, 1.0};
int bounds[4];
int cost;

const int MIN_N = 2;
const int MAX_N = 100;
const string END_COMMAND = "END";

int N;
string rows[MAX_N], contestant_rows[MAX_N];
int sums[MAX_N + 1][MAX_N + 1];

int main(int argc, char *argv[])
{
  if (argc != 3) {
    cerr << "Usage: " << argv[0] << " ./my_solution input_file\n";
    return EXIT_FAILURE;
  }

  ifstream finput(argv[2]);

  if (finput.fail()) {
    cerr << "Error opening input file: " << argv[2] << "\n";
    return EXIT_FAILURE;
  }

  // flog = treasure.log,
  // foutput = stderr
  ofstream flog;
  flog.open("treasure.log");

  ostream &foutput = cerr;

  // Read N.
  if (!(finput >> N)) {
    flog << "Error reading the input file: cannot read integer N." << endl;
    foutput << "Error reading the input file: cannot read integer N." << endl;
    exit(1);
  }

  // Check N.
  if (N < MIN_N || N > MAX_N ) {
    flog << "Error reading the input file: N has to be between " << MIN_N << " and " << MAX_N << "." << endl;
    foutput << "Error reading the input file: N has to be between " << MIN_N << " and " << MAX_N << "." << endl;
    exit(1);
  }
  bounds[0] = f1;
  bounds[1] = f4;
  bounds[2] = f8;
  bounds[3] = f10;

  // Read the map.
  for (int i = 0; i < N; ++i) {
    string row;
    if (!(finput >> row)) {
      flog << "Error reading the input file: cannot read row #" << i << endl;
      foutput << "Error reading the input file: cannot read row #" << i << endl;
      exit(1);
    }
    if (row.size() != N) {
      flog << "Error reading the input file: invalid size of the row #" << i << endl;
      foutput << "Error reading the input file: invalid size of the row #" << i << endl;
      exit(1);
    }
    for (int j = 0; j < N; ++j) {
      if (row[j] != '0' && row[j] != '1') {
        flog << "Error reading the input file: wrong character where '0' or '1' should be." << endl;
        foutput << "Error reading the input file: wrong character where '0' or '1' should be." << endl;
        exit(1);
      }
      // sums are 1-indexed
      sums[i+1][j+1] = sums[i][j+1] + sums[i+1][j] - sums[i][j] + (row[j] == '1');
    }
    rows[i] = row;
  }

  // Check for extra data at the end of the input file.
  string crap;
  if (finput >> crap) {
    flog << "Error reading the input file: Extra data at the end of the input file." << endl;
    foutput << "Error reading the input file: Extra data at the end of the input file." << endl;
    exit(1);
  }

  int p1[2], p2[2];
  pipe(p1);
  pipe(p2);

  pid_t fd = fork();
  if (fd < -1) {
    cerr << "Error with the fork() call." << endl;
    exit(1);
  }

  if (fd == 0) {
    dup2(p1[0], STDIN_FILENO);  close(p1[1]);
    dup2(p2[1], STDOUT_FILENO); close(p2[0]);

    execl(argv[1], argv[1], NULL);
    cerr << "Error executing the contestant program '" << argv[1] << "'.\n";
    exit(1);
  }

  close(p1[0]);
  close(p2[1]);

  dup2(p2[0], STDIN_FILENO);
  dup2(p1[1], STDOUT_FILENO);

  // Input file is valid. Send number N.
  flog << "Input file successfully read (N = " << N << ")." << endl;
  cout << N << endl;

  // Loop.
  while (true)
  {
    string query;
    if (!getline(cin, query)) {
      flog << "Contestant program termimanted exectution without sending a solution." << endl;
      foutput << "Contestant program termimanted exectution without sending a solution." << endl;
      exit(0);
    }
    if (query != END_COMMAND) {
      int r1, c1, r2, c2;
      if (sscanf(query.c_str(), "%d %d %d %d", &r1, &c1, &r2, &c2) != 4) {
        flog << "Invalid question: " << query << endl;
        foutput << "Invalid question: " << query << endl;
	      exit(0);
      }
      if (r1 < 1 || r1 > N) {
        flog << "Invalid question: r1 is not between 1 and N." << endl;
        foutput << "Invalid question: r1 is not between 1 and N." << endl;
        exit(0);
      }
      if (c1 < 1 || c1 > N) {
        flog << "Invalid question: c1 is not between 1 and N." << endl;
        foutput << "Invalid question: c1 is not between 1 and N." << endl;
        exit(0);
      }
      if (r2 < 1 || r2 > N) {
        flog << "Invalid question: r2 is not between 1 and N." << endl;
        foutput << "Invalid question: r2 is not between 1 and N." << endl;
        exit(0);
      }
      if (c2 < 1 || c2 > N) {
        flog << "Invalid question: c2 is not between 1 and N." << endl;
        foutput << "Invalid question: c2 is not between 1 and N." << endl;
        exit(0);
      }
      if (r1 > r2) {
        flog << "Invalid question (r1 > r2)." << endl;
        foutput << "Invalid question (r1 > r2)." << endl;
        exit(0);
      }
      if (c1 > c2) {
        flog << "Invalid question (c1 > c2)." << endl;
        foutput << "Invalid question (c1 > c2)." << endl;
        exit(0);
      }
      flog << query << " => ";

      // Increase cost.
      cost += N * N - (r2 - r1 + 1) * (c2 - c1 + 1) + 1;
      if (cost > bounds[0]) {
        flog << "Too much energy units (more than N^4)." << endl;
        foutput << "Too much energy units (more than N^4)." << endl;
        exit(0);
      }

      // Answer the query.
      int ans = sums[r2][c2] - sums[r1-1][c2] - sums[r2][c1-1] + sums[r1-1][c1-1];
    	flog << ans << endl;
    	cout << ans << endl;

    } else {
      flog << END_COMMAND << " => ";

      // Read the solution.
      for (int i = 0; i < N; ++i) {
        string row;
        if (!(cin >> row)) {
          flog << "Final output is incomplete or missing." << endl;
          foutput << "Final output is incomplete or missing." << endl;
          exit(0);
        }
        if (row.size() != N) {
          flog << "Invalid row size in the final output." << endl;
          foutput << "Invalid row size in the final output." << endl;
          exit(0);
        }
        contestant_rows[i] = row;
      }

      // Check the solution.
      for (int i = 0; i < N; ++i)
        for (int j = 0; j < N; ++j)
          if (contestant_rows[i] != rows[i]) {
            flog << "Final output is incorrect at position (" << i + 1 << ", " << j + 1 << ")." << endl;
            foutput << "Final output is incorrect at position (" << i + 1 << ", " << j + 1 << ")." << endl;
            exit(0);
          }

      float score = 0;
      for (int i = 0; i < 4; ++i)
        if ((double)cost <= bounds[i])
          score = scores[i];

      flog << "Solution is correct. " << cost << " energy units in total. Score = " << (int)(10 * score) << endl;
      foutput << "Solution is correct. " << cost << " energy units in total. Score = " << (int)(10 * score) << endl;
      exit(0);
    }
  }
  return 0;
}
